@@ -54,35 +54,37 @@
 #' @param params A \code{list} with parameters for the model fitting / optimization
 #'             \itemize{
 #'                \item{optimizer_cov}{ Optimizer used for estimating covariance parameters. 
-#'                Options: "gradient_descent", "fisher_scoring", and "nelder_mead". Default = "gradient_descent"}
+#'                Options: "gradient_descent", "fisher_scoring", and "nelder_mead".
+#'                Default= gradient_descent"}
 #'                \item{optimizer_coef}{ Optimizer used for estimating linear regression coefficients, if there are any 
 #'                (for the GPBoost algorithm there are usually none). 
 #'                Options: "gradient_descent", "wls", and "nelder_mead". Gradient descent steps are done simultaneously 
 #'                with gradient descent steps for the covariance parameters. 
 #'                "wls" refers to doing coordinate descent for the regression coefficients using weighted least squares.
-#'                Default="wls" for Gaussian data and "gradient_descent" for other likelihoods.}
-#'                \item{maxit}{ Maximal number of iterations for optimization algorithm. Default=1000.}
+#'                Default="wls" for Gaussian data and "gradient_descent" for other likelihoods}
+#'                \item{maxit}{ Maximal number of iterations for optimization algorithm. Default=1000}
 #'                \item{delta_rel_conv}{ Convergence criterion: stop optimization if relative change 
-#'                in parameters is below this value. Default=1E-6.}
+#'                in parameters is below this value. Default=1E-6}
 #'                \item{init_coef}{ Initial values for the regression coefficients (if there are any, can be NULL).
-#'                Default=NULL.}
+#'                Default=NULL}
 #'                \item{init_cov_pars}{ Initial values for covariance parameters of Gaussian process and 
-#'                random effects (can be NULL). Default=NULL.}
+#'                random effects (can be NULL). Default=NULL}
 #'                \item{lr_coef}{ Learning rate for fixed effect regression coefficients if gradient descent is used.
-#'                Default=0.1.}
+#'                Default=1}
 #'                \item{lr_cov}{ Learning rate for covariance parameters. If <= 0, internal default values are used.
 #'                Default value = 0.1 for "gradient_descent" and 1. for "fisher_scoring"}
 #'                \item{use_nesterov_acc}{ If TRUE Nesterov acceleration is used.
 #'                This is used only for gradient descent. Default=TRUE}
 #'                \item{acc_rate_coef}{ Acceleration rate for regression coefficients (if there are any) 
-#'                for Nesterov acceleration. Default=0.5.}
+#'                for Nesterov acceleration. Default=0.5}
 #'                \item{acc_rate_cov}{ Acceleration rate for covariance parameters for Nesterov acceleration.
-#'                Default=0.5.}
-#'                \item{momentum_offset}{ Number of iterations for which no mometum is applied in the beginning.
-#'                Default=2.}
-#'                \item{trace}{ If TRUE, information on the progress of the parameter optimization is printed. Default=FALSE.}
+#'                Default=0.5}
+#'                \item{momentum_offset}{ Number of iterations for which no momentum is applied in the beginning.
+#'                Default=2}
+#'                \item{trace}{ If TRUE, information on the progress of the parameter
+#'                optimization is printed. Default=FALSE}
 #'                \item{convergence_criterion}{ The convergence criterion used for terminating the optimization algorithm.
-#'                Options: "relative_change_in_log_likelihood" (default) or "relative_change_in_parameters".}
+#'                Options: "relative_change_in_log_likelihood" (default) or "relative_change_in_parameters"}
 #'                \item{std_dev}{ If TRUE, (asymptotic) standard deviations are calculated for the covariance parameters}
 #'            }
 #' @param fixed_effects A \code{vector} of optional external fixed effects which are held fixed during training. 
@@ -627,6 +629,10 @@ gpb.GPModel <- R6::R6Class(
         , convergence_criterion
         , std_dev
       )
+      optim_coef_params <- c("init_coef","lr_coef","acc_rate_coef","optimizer_coef")
+      if (any(names(params) %in% optim_coef_params)) {
+        self$set_optim_coef_params(params=params)
+      }
       return(invisible(self))
     },
     
@@ -659,12 +665,12 @@ gpb.GPModel <- R6::R6Class(
     
     # Set configuration parameters for the optimizer
     set_optim_coef_params = function(params = list()) {
-      num_coef <- NULL
       if (gpb.is.null.handle(private$handle)) {
         stop("GPModel: Gaussian process model has not been initialized")
       }
       private$update_params(params)
       # prepare for calling C++
+      num_coef <- private$num_coef
       init_coef <- private$params[["init_coef"]]
       lr_coef <- private$params[["lr_coef"]]
       acc_rate_coef <- private$params[["acc_rate_coef"]]
@@ -1426,7 +1432,7 @@ gpb.GPModel <- R6::R6Class(
     params = list(maxit = 1000L,
                   delta_rel_conv = 1E-6,
                   init_coef = NULL,
-                  lr_coef = 0.1,
+                  lr_coef = 1.,
                   lr_cov = -1.,
                   use_nesterov_acc = TRUE,
                   acc_rate_coef = 0.5,
