@@ -12,25 +12,25 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
   }
   
   # Create data
-  n <- 10 # number of samples
+  n <- 1000 # number of samples
   # First grouped random effects model
-  m <- 5 # number of categories / levels for grouping variable
+  m <- 100 # number of categories / levels for grouping variable
   group <- rep(1,n) # grouping variable
   for(i in 1:m) group[((i-1)*n/m+1):(i*n/m)] <- i
   Z1 <- model.matrix(rep(1,n) ~ factor(group) - 1)
-  b1 <- 1:m-3
-  # Second random effect
-  n_obs_gr <- n/m # number of samples per group
+  b1 <- qnorm(sim_rand_unif(n=m, init_c=0.546))
+  # Second random effecs
+  n_gr <- n/20 # number of groups
   group2 <- rep(1,n) # grouping variable
-  for(i in 1:m) group2[(1:n_obs_gr)+n_obs_gr*(i-1)] <- 1:n_obs_gr
+  for(i in 1:(n/n_gr)) group2[(1:n_gr)+n_gr*(i-1)] <- 1:n_gr
   Z2 <- model.matrix(rep(1,n)~factor(group2)-1)
-  b2 <- c(-1,1)
+  b2 <- qnorm(sim_rand_unif(n=length(unique(group2)), init_c=0.46))
   # Random slope / coefficient
   x <- cos((1:n-n/2)^2*5.5*pi/n) # covariate data for random slope
   Z3 <- diag(x) %*% Z1
-  b3 <- sin((1:m-3))^2/3
+  b3 <- qnorm(sim_rand_unif(n=m, init_c=0.69))
   # Error term
-  xi <- qnorm(sim_rand_unif(n=n, init_c=0.1)) / 50
+  xi <- sqrt(0.5) * qnorm(sim_rand_unif(n=n, init_c=0.1))
   # Data for linear mixed effects model
   X <- cbind(rep(1,n),sin((1:n-n/2)^2*2*pi/n)) # design matrix / covariate data for fixed effect
   beta <- c(2,2) # regression coefficients
@@ -44,11 +44,11 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- GPModel(group_data = group)
     fit(gp_model, y = y, params = list(std_dev = TRUE, optimizer_cov = "fisher_scoring",
                                        convergence_criterion = "relative_change_in_parameters"))
-    cov_pars <- c(0.0006328951, 0.0004002780, 2.0227167347, 1.2794785465)
+    cov_pars <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
     expect_equal(dim(gp_model$get_cov_pars())[2], 2)
     expect_equal(dim(gp_model$get_cov_pars())[1], 2)
-    expect_equal(gp_model$get_num_optim_iter(), 13)
+    expect_equal(gp_model$get_num_optim_iter(), 6)
     # Using gradient descent instead of Fisher scoring
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
@@ -58,7 +58,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expect_lt(sum(abs(cov_pars_est-cov_pars[c(1,3)])),1E-5)
     expect_equal(class(cov_pars_est), "numeric")
     expect_equal(length(cov_pars_est), 2)
-    expect_equal(gp_model$get_num_optim_iter(), 124)
+    expect_equal(gp_model$get_num_optim_iter(), 9)
     # Using gradient descent with Nesterov acceleration
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
@@ -66,39 +66,39 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          acc_rate_cov = 0.1, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),1E-5)
-    expect_equal(gp_model$get_num_optim_iter(), 55)
+    expect_equal(gp_model$get_num_optim_iter(), 10)
     # Using gradient descent and a too large learning rate
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
                                          lr_cov = 10, use_nesterov_acc = FALSE,
                                          maxit = 1000, convergence_criterion = "relative_change_in_parameters"))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 27)
+    expect_equal(gp_model$get_num_optim_iter(), 32)
     # Different termination criterion
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_log_likelihood"))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 12)
+    expect_equal(gp_model$get_num_optim_iter(), 5)
     # Nelder-Mead
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = TRUE))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-3)
-    expect_equal(gp_model$get_num_optim_iter(), 18)
+    expect_equal(gp_model$get_num_optim_iter(), 12)
     
     # Evaluation of likelihood
     ll <- gp_model$neg_log_likelihood(y=y,cov_pars=gp_model$get_cov_pars()[1,])
-    expect_lt(abs(ll-(-0.7292666)),1E-6)
+    expect_lt(abs(ll-(1228.293)),1E-2)
     
     # Prediction 
     gp_model <- GPModel(group_data = group)
-    group_test <- c(1,2,7)
+    group_test <- c(1,2,m+1)
     expect_error(predict(gp_model, y=y, cov_pars = c(0.5,1.5)))# group data not provided
     pred <- predict(gp_model, y=y, group_data_pred = group_test,
                     cov_pars = c(0.5,1.5), predict_cov_mat = TRUE)
-    expected_mu <- c(-1.7303927, -0.8722256,  0.0000000)
-    expected_cov <- c(0.7142857, 0.0000000, 0.0000000, 0.0000000,
-                      0.7142857, 0.0000000, 0.0000000, 0.0000000, 2.0000000)
+    expected_mu <- c(-0.1553877, -0.3945731, 0)
+    expected_cov <- c(0.5483871, 0.0000000, 0.0000000, 0.0000000,
+                      0.5483871, 0.0000000, 0.0000000, 0.0000000, 2)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
     expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
     # Predict variances
@@ -111,32 +111,33 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "fisher_scoring",
                                          convergence_criterion = "relative_change_in_parameters"))
-    group_test <- c(1,2,7)
+    group_test <- c(1,2,m+1)
     pred <- predict(gp_model, group_data_pred = group_test, predict_cov_mat = TRUE)
-    expected_mu <- c(-2.018476, -1.017437, 0.0000000)
-    expected_cov <- c(0.0009492931, 0.0000000000, 0.0000000000, 0.0000000000,
-                      0.0009492931, 0.0000000000, 0.0000000000, 0.0000000000, 2.0233496298)
+    expected_mu <- c(-0.1543396, -0.3919117, 0.0000000)
+    expected_cov <- c(0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000,
+                      0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000, 1.7164805)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
     expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1),y=y)
-    expect_lt(abs(nll-10.12033),1E-5)
+    expect_lt(abs(nll-2282.073),1E-2)
     
     # Do optimization using optim and e.g. Nelder-Mead
     gp_model <- GPModel(group_data = group)
     opt <- optim(par=c(1,1), fn=gp_model$neg_log_likelihood, y=y, method="Nelder-Mead")
     expect_lt(sum(abs(opt$par-cov_pars[c(1,3)])),1E-3)
-    expect_lt(abs(opt$value-(-0.7292666)),1E-5)
-    expect_equal(as.integer(opt$counts[1]), 145)
+    expect_lt(abs(opt$value-(1228.293)),1E-2)
+    expect_equal(as.integer(opt$counts[1]), 49)
     
     # Use non-ordered grouping data
-    shuffle_ind <- c(10, 2, 1, 9, 7, 8, 4, 5, 3, 6)
+    set.seed(1)
+    shuffle_ind <- sample.int(n=n,size=n,replace=FALSE)
     gp_model <- GPModel(group_data = group[shuffle_ind])
     fit(gp_model, y = y[shuffle_ind], params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                                     convergence_criterion = "relative_change_in_parameters"))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 13)
+    expect_equal(gp_model$get_num_optim_iter(), 6)
   })
   
   
@@ -149,46 +150,45 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "fisher_scoring",
                                          optimizer_coef = "wls", std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
-    cov_pars <- c(0.0005890585, 0.0003725533, 2.0227180604, 1.2794655172)
-    coef <- c(1.99219041, 0.63608373, 2.01453829, 0.02383676)
+    cov_pars <- c(0.49205230, 0.02319557, 1.22064076, 0.17959832)
+    coef <- c(2.07499902, 0.11269252, 1.94766255, 0.03382472)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
     expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 13)
+    expect_equal(gp_model$get_num_optim_iter(), 7)
     
     # Prediction
-    group_test <- c(1,2,7)
+    group_test <- c(1,2,m+1)
     X_test <- cbind(rep(1,3),c(-0.5,0.2,0.4))
     expect_error(predict(gp_model,group_data_pred = group_test))# covariate data not provided
     pred <- predict(gp_model, group_data_pred = group_test,
                     X_pred = X_test, predict_cov_mat = TRUE)
-    expected_mu <- c(-1.017224, 1.376914, 2.798006)
-    expected_cov <- c(0.0008835448, 0.0000000000, 0.0000000000, 0.0000000000,
-                      0.0008835448, 0.0000000000, 0.0000000000, 0.0000000000, 2.0233071188)
+    expected_mu <- c(0.886494, 2.043259, 2.854064)
+    expected_cov <- c(0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000,
+                      0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000, 1.712693)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
     expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
     
     # Fit model using gradient descent instead of wls for regression coefficients
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
-                           params = list(optimizer_cov = "fisher_scoring", maxit=10000, std_dev = TRUE,
-                                         optimizer_coef = "gradient_descent", lr_coef=0.001, use_nesterov_acc=FALSE,
-                                         convergence_criterion = "relative_change_in_parameters"))
-    cov_pars <- c(0.0005890585, 0.0003725533, 2.0227193907, 1.2794663580)
-    coef <- c(1.99104872, 0.63608394, 2.01453829, 0.02383676)
+                           params = list(optimizer_cov = "gradient_descent", maxit=1000, std_dev = TRUE,
+                                         optimizer_coef = "gradient_descent", lr_coef=1, use_nesterov_acc=TRUE))
+    cov_pars <- c(0.49205012, 0.02319547, 1.22089504, 0.17963425)
+    coef <- c(2.07513927, 0.11270379, 1.94322756, 0.03382466)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
     expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 3437)
+    expect_equal(gp_model$get_num_optim_iter(), 110)
     
     # Fit model using Nelder-Mead
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "nelder_mead",
-                                         optimizer_coef = "gradient_descent", std_dev = TRUE))
-    cov_pars <- c(0.0005892916, 0.0003727007, 2.0219180842, 1.2789596377)
-    coef <- c(1.99260508, 0.63595798, 2.01454390, 0.02384147)
+                                         optimizer_coef = "gradient_descent", lr_coef=0.1, std_dev = TRUE))
+    cov_pars <- c(0.49175276, 0.02318145, 1.22767304, 0.18058850)
+    coef <- c(2.07357920, 0.11300277, 1.94743042, 0.03381487)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
     expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 198)
+    expect_equal(gp_model$get_num_optim_iter(), 171)
   })
   
   
@@ -202,32 +202,31 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=5, std_dev = TRUE))
-    expected_values <- c(0.01328599, 0.05137801, 2.42699005, 1.70170676,
-                         1.23536565, 1.54308011, 0.10065015, 0.12494428)
+    expected_values <- c(0.49554952, 0.02546769, 1.24880860, 0.18983953, 1.05505134, 0.22337199, 1.13840014, 0.17950490)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
     expect_equal(gp_model$get_num_optim_iter(), 5)
     
     # Prediction
     gp_model <- GPModel(group_data = cbind(group,group2),
                         group_rand_coef_data = x, ind_effect_group_rand_coef = 1)
-    group_data_pred = cbind(c(1,1,7),c(2,1,3))
+    group_data_pred = cbind(c(1,1,m+1),c(2,1,length(group2)+1))
     group_rand_coef_data_pred = c(0,0.1,0.3)
     expect_error(gp_model$predict(group_data_pred = group_data_pred,
                                   cov_pars = c(0.1,1,2,1.5), y=y))# random slope data not provided
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
                              group_rand_coef_data_pred=group_rand_coef_data_pred,
                              cov_pars = c(0.1,1,2,1.5), predict_cov_mat = TRUE)
-    expected_mu <- c(-0.01123019, -1.88978313, 0.000000000)
-    expected_cov <- c(0.8005528, 0.6179886, 0.0000000, 0.6179886,
-                      0.7798839, 0.0000000, 0.0000000, 0.0000000, 3.2350000)
+    expected_mu <- c(0.7579961, -0.3755475, 0.000000000)
+    expected_cov <- c(0.115340856 , 0.009310105 , 0.0000000, 0.009310105 ,
+                      0.115340856 , 0.0000000, 0.0000000, 0.0000000, 3.235)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-3)
     # Predict variances
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
                              group_rand_coef_data_pred=group_rand_coef_data_pred,
                              cov_pars = c(0.1,1,2,1.5), predict_var = TRUE)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),1E-6)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),1E-3)
     
     # Gradient descent
     gp_model <- fitGPModel(group_data = cbind(group,group2),
@@ -235,8 +234,9 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-c(0.0002845659, 2.4833741395, 1.2376189948, 0.0428108436 ))),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(),1000)
+    cov_pars <- c(0.4958303, 1.2493181, 1.0543343, 1.1388604 )
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_equal(gp_model$get_num_optim_iter(),8)
     
     # Nelder-Mead
     gp_model <- fitGPModel(group_data = cbind(group,group2),
@@ -244,11 +244,12 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = FALSE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-c(0.0522126336, 2.4287341933, 1.2126494963, 0.0001102573))),1E-6)
+    cov_pars <- c(0.4959521, 1.2408579, 1.0560989, 1.1373089 )
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1,2,1.5),y=y)
-    expect_lt(abs(nll-18.06998),1E-5)
+    expect_lt(abs(nll-2335.803),1E-2)
   })
   
   
@@ -259,9 +260,9 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=100, std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
-    expected_values <- c(0.0006328951, 0.0004002780, 2.0227167465, 1.2794785483)
+    expected_values <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 13)
+    expect_equal(gp_model$get_num_optim_iter(), 6)
     
     # gradient descent
     gp_model <- fitGPModel(group_data = group, cluster_ids = cluster_ids,
@@ -269,13 +270,12 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "gradient_descent", std_dev = TRUE,
                                          lr_cov = 0.1, use_nesterov_acc = FALSE, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
-    expected_values <- c(0.0006328973, 0.0004002794, 2.0227113725, 1.2794751488)
-    cov_pars_est <- gp_model$get_cov_pars()
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 124)
+    cov_pars_expected <- c(0.49348532, 0.02326312, 1.22299520, 0.17995161)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),1E-6)
+    expect_equal(gp_model$get_num_optim_iter(), 9)
     
     # Prediction
-    group_data_pred = c(1,1,7)
+    group_data_pred = c(1,1,m+1)
     cluster_ids_pred = c(1,3,1)
     gp_model <- GPModel(group_data = group, cluster_ids = cluster_ids)
     expect_error(gp_model$predict(group_data_pred = group_data_pred,
@@ -283,8 +283,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred,
                              cov_pars = c(0.75,1.25), predict_cov_mat = TRUE)
-    expected_mu <- c(-1.552917, 0.000000, 0.000000)
-    expected_cov <- c(1.038462, 0.000000, 0.000000, 0.000000,
+    expected_mu <- c(-0.1514786, 0.000000, 0.000000)
+    expected_cov <- c(0.8207547, 0.000000, 0.000000, 0.000000,
                       2.000000, 0.000000, 0.000000, 0.000000, 2.000000)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
     expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
@@ -295,18 +295,14 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=100, std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
-    expected_values <- c(0.0006328951, 0.0004002780, 2.0227167465, 1.2794785483)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 13)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),1E-6)
+    expect_equal(gp_model$get_num_optim_iter(), 6)
     # Prediction
-    group_data_pred = c(1,1,7)
+    group_data_pred = c(1,1,m+1)
     cluster_ids_pred_string = paste0(as.character(c(1,3,1)),"_s")
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred_string,
                              cov_pars = c(0.75,1.25), predict_cov_mat = TRUE)
-    expected_mu <- c(-1.552917, 0.000000, 0.000000)
-    expected_cov <- c(1.038462, 0.000000, 0.000000, 0.000000,
-                      2.000000, 0.000000, 0.000000, 0.000000, 2.000000)
     expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
     expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
 
@@ -320,9 +316,9 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
                                          lr_cov = 0.1, use_nesterov_acc = TRUE, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
-    expected_values <- c(3.431451e-05, 2.075518e+00, 4.309345e-02)
+    expected_values <- c(0.4927786, 1.2565095, 1.1333656)
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
-    expect_equal(gp_model$get_num_optim_iter(), 1000)
+    expect_equal(gp_model$get_num_optim_iter(), 14)
     
   })
   
